C     munfill.f -- unfold radar velocity data
C
C     Original mex file by Danny Chapman
C
C     Incorporated into the chil code by Robin Hogan from 2 Nov 2001
C
C============== MUNFILL SUBROUTINE =============================
C vel - output velocity field
C region - number to indicate which region each pixel is in
C v_orig - original velocity data
C [mm,nn] - the size of the arrays vel, region and v_orig
C vf - folding velocity goes from -vf to +vf
C nan - value to indicate no data
C===============================================================
      subroutine munfill(vel,region,v_orig,mm,nn,vf,nan,isppi
     $     ,isreversed)
      IMPLICIT NONE
      integer*4 mm, nn, M, N, isppi,isreversed
      PARAMETER (M=400, N=1600)

      integer*4 i, j,ii,jj,k,l,o,p,stacki(M*N), stackj(M*N), numreg(M
     $     *N),mask(M,N),reg,nor,mink,minl
      integer*4 stackindex, goon, flagup1, flagdown1,region_num,
     $     stacksize, stackreg(M*N), max_reg

      integer*4 region(mm,nn)

      real*4 vel(mm, nn), v_orig(mm, nn),v(M,N),av

      real*4 nan
      real*4 vf, vf2, dist_min,dist,change,maxvar,lonely1

      vf2 = vf*2.0
      change = 0.0
      
      IF ((mm.LE.M).AND.(nn.LE.N)) THEN
      
C     unfolds by filling in regions...

C     first zap the data

         maxvar=4.0
         lonely1=4.0

         write(*,'(A,$)') 'Zapping...'
         
         CALL mzap(vel,v_orig,maxvar,lonely1,0.0,mm,nn,vf,nan)

C and reduce the complexity by removing small regions...

         if (nn.gt.300) then
            write(*,'(A,$)') ' removing small regions...'
            CALL reduce(vel,10,mm,nn,nan)
         endif

C Start the calculation...
         DO i=1,mm
            DO j=1,nn
               v(i,j)=vel(i,j)
               vel(i,j)=v(i,j)
               mask(i,j)=0      ! haven't unfolded yet
               region(i,j)=0    ! this will be the default undefined region
            ENDDO
         ENDDO

         region_num=1

C     Find a starting pixel, assume it does not need unfolding, and
C     work from there.  Note that this bit is rather crucial as if you
C     choose the wrong point then the whole field will be wrong.
         IF (isppi) THEN
            IF (isreversed) THEN
C     Can't imagine this scenario ever occurring:
C     We're searching all the data at the highest gate, then all the
c     data at the next highest...
               DO i=mm,1,-1
                  DO j=1,nn
                     IF (v(i,j).NE.nan) THEN
                        GOTO 10
                     ENDIF
                  ENDDO
               ENDDO
            ELSE
C     The normal PPI: search 1st gate, all rays; 2nd gate, all rays etc.
               DO i=1,mm
                  DO j=1,nn
                     IF (v(i,j).NE.nan) THEN
                        GOTO 10
                     ENDIF
                  ENDDO
               ENDDO
            ENDIF
         ELSE
            IF (isreversed) THEN
C     An RHI that goes from a high elevation to a low elevation
               DO j=nn,1,-1
                  DO i=1,mm
                     IF (v(i,j).NE.nan) THEN
                        GOTO 10
                     ENDIF
                  ENDDO
               ENDDO
            ELSE
C     An RHI that goes from a low elevation to a high elevation
               DO j=1,nn
                  DO i=1,mm
                     IF (v(i,j).NE.nan) THEN
                        GOTO 10
                     ENDIF
                  ENDDO
               ENDDO
            ENDIF
         ENDIF


C     Found our starting point...
C     Loop below jumps up to this point after finding a starting i and
c     j...
 10      stackindex=1
         stacki(stackindex)=i
         stackj(stackindex)=j

         DO WHILE (stackindex.GT.0)
	
            i=stacki(stackindex)
            j=stackj(stackindex)

 
            stackindex=stackindex-1
	
C     Check to see if this line has already been done

            IF (v(i,j).NE.nan) THEN

C     let's do it...
C     first move to the beginning of the line
C     but... we need to unfold as we go... this difference is that
c     here we don't set v to NaN, so when we reach the end, we come 
c     back
	
               call unf(vel,mask,region,region_num,i,j,mm,nn,M,N,vf)
               mask(i,j)=1
               region(i,j)=region_num

               goon=1
               DO WHILE ((i .GT. 1).AND.(goon.EQ.1))
                  IF (v(i-1,j).NE.nan) THEN  
                     i=i-1
                     call unf(vel,mask,region,region_num,i,j,mm,nn,M,N
     $                    ,vf)
                     mask(i,j)=1
                     region(i,j)=region_num
                  else
                     goon=0
                  endif
               enddo
		
C     mark this location

               v(i,j)=nan
               call unf(vel,mask,region,region_num,i,j,mm,nn,M,N,vf)
               mask(i,j)=1
               region(i,j)=region_num

               flagup1=1        ! "fix" the flags for previous nans...
               flagdown1=1      ! 1=NaN, 0=clear
		
               goon=1
		
C     move along...
               do while ((i .lt. mm).and.(goon.eq.1))
C     check if clear above...
                  if (j .lt. nn) then 
                     if ((v(i,j+1).ne.nan).and.(flagup1.eq.1))
     $                    then
                        stackindex=stackindex+1
                        stacki(stackindex)=i
                        stackj(stackindex)=j+1
                     endif
                     flagup1=(v(i,j+1).eq.nan)
                  endif
                  if (j .gt. 1) then 
                     if ((v(i,j-1).ne.nan).and.(flagdown1.eq.1
     $                    )) then
                        stackindex=stackindex+1
                        stacki(stackindex)=i
                        stackj(stackindex)=j-1
                     endif
                     flagdown1=(v(i,j-1).eq.nan)
                  endif
			
                  if (v(i+1,j).ne.nan) then
                     i=i+1
                     call unf(vel,mask,region,region_num,i,j,mm,nn,M,N
     $                    ,vf)
                     mask(i,j)=1
                     v(i,j)=nan
                     region(i,j)=region_num
                  else
                     goon = 0
                  endif
		
               enddo

            endif
		
         enddo
C     Check for any more regions containing data

C     Find a starting pixel, assume it does not need unfolding, and
C     work from there.  Note that this bit is rather crucial as if you
C     choose the wrong point then the whole field will be wrong.
         IF (isppi) THEN
            IF (isreversed) THEN
C     Can't imagine this scenario ever occurring:
C     We're searching all the data at the highest gate, then all the
c     data at the next highest...
               DO i=mm,1,-1
                  DO j=1,nn
                     IF (v(i,j).NE.nan) THEN
                        region_num=region_num+1
                       GOTO 10
                     ENDIF
                  ENDDO
               ENDDO
            ELSE
C     The normal PPI: search 1st gate, all rays; 2nd gate, all rays etc.
               DO i=1,mm
                  DO j=1,nn
                     IF (v(i,j).NE.nan) THEN
                        region_num=region_num+1
                        GOTO 10
                     ENDIF
                  ENDDO
               ENDDO
            ENDIF
         ELSE
            IF (isreversed) THEN
C     An RHI that goes from a high elevation to a low elevation
               DO j=nn,1,-1
                  DO i=1,mm
                     IF (v(i,j).NE.nan) THEN
                        region_num=region_num+1
                        GOTO 10
                     ENDIF
                  ENDDO
               ENDDO
            ELSE
C     An RHI that goes from a low elevation to a high elevation
               DO j=1,nn
                  DO i=1,mm
                     IF (v(i,j).NE.nan) THEN
                        region_num=region_num+1
                       GOTO 10
                     ENDIF
                  ENDDO
               ENDDO
            ENDIF
         ENDIF
             
         nor=region_num ! number of regions
         write(*,'(I4,A,$)') nor, ' regions found...'
C     At this point, we should have unfolded all the individual
C     regions, and worked out in which region each point lies. Now we
C     want to combine regions together... Start with region one, and
C     merge it with the clostest region... repeat until we just have
C     one region left.

C     Count the region sizes...

         do i=1,nor
            numreg(i)=0
         enddo
         
         DO i=1,mm
            DO j=1,nn
               if (region(i,j).ne.0) then
                  numreg(region(i,j))=numreg(region(i,j))+1
               endif
            enddo
         enddo

C Get rid of really small pathetic regions...

         if (nn.gt.400) then
            max_reg=0
            do k=1,nor
               if (numreg(k).gt.max_reg) then
                  max_reg=numreg(k)
               endif
            enddo
            max_reg=max_reg/400
         else
            max_reg=6
         endif
         
         goon=0
         do k=1,nor
            if (numreg(k).le.max_reg) then
               do i=1,mm
                  do j=1,nn
                     if (region(i,j).eq.k) then
                        vel(i,j)=nan
                        region(i,j)=0
                     endif
                  enddo
               enddo
               numreg(k)=0
               goon=goon+1
            endif
         enddo

         write(*,'(I4,A,$)') nor-goon, ' to be unfolded'

C     Have it so that we output the regions too - note we don't need
c     mask any more so we can overwrite and use it
C     Also, when we start looking for nearest points, we only want to look
c     at points on boundaries (as interior points will never be
c     closest), so calculate these locations in advance

         stackindex=0
         do i=2,mm-1
            do j=2,nn-1
               mask(i,j)=region(i,j)
               if (vel(i,j).ne.nan) then
                  k=(vel(i-1,j+1).eq.nan)+(vel(i,j+1).eq.nan) +(vel(i+1
     $                 ,j+1).eq.nan)+(vel(i-1 ,j).eq.nan)+(vel(i +1,j)
     $                 .eq.nan)+(vel(i-1,j-1).eq.nan)+(vel(i,j -1).eq
     $                 .nan)+(vel(i+1,j -1).eq.nan)
                  if (k.gt.0) then
                     stackindex=stackindex+1
                     stacki(stackindex)=i
                     stackj(stackindex)=j
                     stackreg(stackindex)=region(i,j)
                  endif
               endif
            enddo
         enddo
         stacksize=stackindex

C     find the closest region to the region which contains the most
C     points (see above)...
         max_reg=0
         do k=1,nor
            if (numreg(k).gt.max_reg) then
               max_reg=numreg(k)
               region_num=k
            endif
         enddo

         do o=1,nor-1
            if (numreg(o).gt.0) then
               dist_min=1E10
               mink=0
               minl=0
               do k=1,stacksize
                  if (stackreg(k).ne.region_num) then
                     do l=1,stacksize
                        if (stackreg(l).eq.region_num) then
                           dist=(stacki(k)-stacki(l))**2.0+(stackj(k)
     $                          -stackj(l))**2.0
                           if (dist.lt.dist_min) then
                              dist_min=dist
                              mink=k
                              minl=l
                              if (dist.lt.5) then
                                 goto 50
                              endif
                           endif
                        endif
                     enddo
                  endif
               enddo
 50            if (mink.gt.0) then
                  i=stacki(mink) ! this is the one we're going to change
                  j=stackj(mink)
                  ii=stacki(minl) ! this is the one that is in region 1
                  jj=stackj(minl)
                  reg=stackreg(mink)

C     Now we should have found the point closest to ii,jj, so work out
C     what velocity fold the new region needs...
                  if ((vel(i,j)-vel(ii,jj)).gt.(2.5*vf2)) then
                     change=-3.0*vf2
                  elseif ((vel(i,j)-vel(ii,jj)).gt.(1.5*vf2)) then
                     change=-2.0*vf2
                  elseif ((vel(i,j)-vel(ii,jj)).gt.(0.5*vf2)) then
                     change=-1.0*vf2
                  elseif ((vel(i,j)-vel(ii,jj)).gt.(-0.5*vf2)) then
                     change=-0.0*vf2
                  elseif ((vel(i,j)-vel(ii,jj)).gt.(-1.5*vf2)) then
                     change=1.0*vf2
                  elseif ((vel(i,j)-vel(ii,jj)).gt.(-2.5*vf2)) then
                     change=2.0*vf2
                  elseif ((vel(i,j)-vel(ii,jj)).gt.(-3.5*vf2)) then
                     change=3.0*vf2
                  endif
C     Now add change to all the points in this second region, and
C     change the region array so that it is equal to region_num  
                  write(*,'(A, $)') '.'
!                  print#, 'Number: ',o,'   Region: ',reg,'  Points: ',
!     $                 numreg(stackreg(mink)), '  Change: ',change,
!     $                 vf2
                 
                  DO i=1,mm
                     DO j=1,nn
                        if (mask(i,j).eq.reg) then
                           vel(i,j)=vel(i,j)+change
                           mask(i,j)=1
                        endif
                     enddo
                  enddo
                  
                  do p=1,stacksize
                     if (stackreg(p).eq.reg) then
                        stackreg(p)=region_num
                     endif
                  enddo
               endif      
            endif
         enddo 
            
C     now we've unfolded all the good data - use this as a "mask" to
C     unfold everything else too (in case we chopped out some good
C     data)
         write(*,'(A,$)') ' replacing zapped data...'
         CALL replace(vel,mm,nn,nan)

C     now treat vel as just a mask for the original velocity array -
C     ie fold the original to get it as close as possible to the mask
         
         do j=1,nn
            do i=1,mm
               
               av=vel(i,j)
               vel(i,j)=v_orig(i,j)
               if ((vel(i,j)-av).gt.(2.5*vf2)) then
                  vel(i,j)=vel(i,j)-3.0*vf2
               elseif ((vel(i,j)-av).gt.(1.5*vf2)) then
                  vel(i,j)=vel(i,j)-2.0*vf2
               elseif ((vel(i,j)-av).gt.(0.5*vf2)) then
                  vel(i,j)=vel(i,j)-1.0*vf2
               elseif ((vel(i,j)-av).gt.(-0.5*vf2)) then
                  vel(i,j)=vel(i,j)-0.0*vf2
               elseif ((vel(i,j)-av).gt.(-1.5*vf2)) then
                  vel(i,j)=vel(i,j)+1.0*vf2
               elseif ((vel(i,j)-av).gt.(-2.5*vf2)) then
                  vel(i,j)=vel(i,j)+2.0*vf2
               elseif ((vel(i,j)-av).gt.(-3.5*vf2)) then
                  vel(i,j)=vel(i,j)+3.0*vf2
               endif
               
            enddo
         enddo
         
      ELSE                      !mm and/or nn too big
         
         PRINT*, 'Input matrix is too large (mm=', mm, ' nn=', nn,
     $        ' M=', M, ' N=', N, ')'
         DO i=1,mm
            DO j=1,nn
               vel(i,j)=v(i,j)
            ENDDO
         ENDDO
         
      ENDIF
      
 
      return
      end

C====================================================================

      subroutine unf(vel,mask,region,region_num,i,j,mm,nn,M,N,vf)
      IMPLICIT NONE
      integer*4 mm, nn,M,N

      integer*4 i, j, ii,jj, region_num,mask(M,N), count

      integer*4 region(mm,nn)

      real*4 vel(mm, nn)

      real*4 total,av
      real*4 vf, vf2

      vf2 = vf * 2.0

C     work out the average of the valid (i.e. already unfolded)
C     velocities

      count=0
      total=0.0

      ii=i-1
      jj=j+1
      if ((ii.GE.1).AND.(ii.LE.mm).AND.(jj.GE.1).AND.(jj.LE.nn).AND
     $     .(region(ii,jj).EQ.region_num)) then
         total=total+mask(ii,jj)*vel(ii,jj)
         count=count+mask(ii,jj)
      endif
      ii=i
      jj=j+1
      if ((ii.GE.1).AND.(ii.LE.mm).AND.(jj.GE.1).AND.(jj.LE.nn).AND
     $     .(region(ii,jj).EQ.region_num)) then
         total=total+mask(ii,jj)*vel(ii,jj)
         count=count+mask(ii,jj)
      endif
      ii=i+1
      jj=j+1
      if ((ii.GE.1).AND.(ii.LE.mm).AND.(jj.GE.1).AND.(jj.LE.nn).AND
     $     .(region(ii,jj).EQ.region_num)) then
         total=total+mask(ii,jj)*vel(ii,jj)
         count=count+mask(ii,jj)
      endif
      ii=i+1
      jj=j
      if ((ii.GE.1).AND.(ii.LE.mm).AND.(jj.GE.1).AND.(jj.LE.nn).AND
     $     .(region(ii,jj).EQ.region_num)) then
         total=total+mask(ii,jj)*vel(ii,jj)
         count=count+mask(ii,jj)
      endif
      ii=i+1
      jj=j-1
      if ((ii.GE.1).AND.(ii.LE.mm).AND.(jj.GE.1).AND.(jj.LE.nn).AND
     $     .(region(ii,jj).EQ.region_num)) then
         total=total+mask(ii,jj)*vel(ii,jj)
         count=count+mask(ii,jj)
      endif
      ii=i
      jj=j-1
      if ((ii.GE.1).AND.(ii.LE.mm).AND.(jj.GE.1).AND.(jj.LE.nn).AND
     $     .(region(ii,jj).EQ.region_num)) then
         total=total+mask(ii,jj)*vel(ii,jj)
         count=count+mask(ii,jj)
      endif
      ii=i-1
      jj=j-1
      if ((ii.GE.1).AND.(ii.LE.mm).AND.(jj.GE.1).AND.(jj.LE.nn).AND
     $     .(region(ii,jj).EQ.region_num)) then
         total=total+mask(ii,jj)*vel(ii,jj)
         count=count+mask(ii,jj)
      endif
      ii=i-1
      jj=j
      if ((ii.GE.1).AND.(ii.LE.mm).AND.(jj.GE.1).AND.(jj.LE.nn).AND
     $     .(region(ii,jj).EQ.region_num)) then
         total=total+mask(ii,jj)*vel(ii,jj)
         count=count+mask(ii,jj)
      endif

      if (count.gt.0) then
         av=total/count
         if ((vel(i,j)-av).gt.(2.5*vf2)) then
            vel(i,j)=vel(i,j)-3.0*vf2
         elseif ((vel(i,j)-av).gt.(1.5*vf2)) then
            vel(i,j)=vel(i,j)-2.0*vf2
         elseif ((vel(i,j)-av).gt.(0.5*vf2)) then
            vel(i,j)=vel(i,j)-1.0*vf2
         elseif ((vel(i,j)-av).gt.(-0.5*vf2)) then
            vel(i,j)=vel(i,j)-0.0*vf2
         elseif ((vel(i,j)-av).gt.(-1.5*vf2)) then
            vel(i,j)=vel(i,j)+1.0*vf2
         elseif ((vel(i,j)-av).gt.(-2.5*vf2)) then
            vel(i,j)=vel(i,j)+2.0*vf2
         elseif ((vel(i,j)-av).gt.(-3.5*vf2)) then
            vel(i,j)=vel(i,j)+3.0*vf2
         endif
      endif ! otherwise just leave vel alone

      return
      end



C=========================================================
C     fills in regions of NaNs                            
C=========================================================
      subroutine replace(vel,mm,nn,nan)
      IMPLICIT NONE

      integer*4 mm, nn, M, N
      integer*4 i, j,k, count,ii,jj

      PARAMETER (M=400, N=1600)
      real*4 vel(mm,nn), v(M, N), nan
      integer*4 stacki(M*N), stackj(M*N),stackindex,stacki2(M*N),
     $     stackj2(M*N),stackcount,stacksize

C     Find all the points which are on the edge of regions - we only
C     want to spread out from these

      stackindex=0
      do i=2,mm-1
         do j=2,nn-1
            v(i,j)=vel(i,j)
            if (vel(i,j).ne.nan) then
               k=(vel(i-1,j+1).eq.nan)+(vel(i,j+1).eq.nan)+(vel(i+1,j+1)
     $              .eq.nan)+(vel(i-1,j).eq.nan)+(vel(i+1,j).eq.nan)
     $              +(vel(i-1,j-1).eq.nan)+(vel(i,j-1).eq.nan)+(vel(i+1
     $              ,j-1).eq.nan)
               if (k.gt.0) then
                  stackindex=stackindex+1
                  stacki(stackindex)=i
                  stackj(stackindex)=j
               endif
            endif
         enddo
      enddo
      stacksize=stackindex

      count=stacksize
      do while (count.gt.0)
C      do k=1,2
         stackcount=0
         do stackindex=1,count
            i=stacki(stackindex)
            j=stackj(stackindex)
            if ((i.gt.1).and.(i.lt.mm).and.(j.gt.1).and.(j.lt.nn))
     $           then  
               
               ii=i-1
               jj=j+1
               if (v(ii,jj).eq.nan) then
                  v(ii,jj)=vel(i,j)
                  stackcount=stackcount+1
                  stacki2(stackcount)=ii
                  stackj2(stackcount)=jj
               endif
               ii=i
               jj=j+1
               if (v(ii,jj).eq.nan) then
                  v(ii,jj)=vel(i,j)
                  stackcount=stackcount+1
                  stacki2(stackcount)=ii
                  stackj2(stackcount)=jj
               endif
               ii=i-1
               jj=j+1
               if (v(ii,jj).eq.nan) then
                  v(ii,jj)=vel(i,j)
                  stackcount=stackcount+1
                  stacki2(stackcount)=ii
                  stackj2(stackcount)=jj
               endif
               ii=i+1
               jj=j+1
               if (v(ii,jj).eq.nan) then
                  v(ii,jj)=vel(i,j)
                  stackcount=stackcount+1
                  stacki2(stackcount)=ii
                  stackj2(stackcount)=jj
               endif
               ii=i-1
               jj=j+1
               if (v(ii,jj).eq.nan) then
                   v(ii,jj)=vel(i,j)
                  stackcount=stackcount+1
                  stacki2(stackcount)=ii
                  stackj2(stackcount)=jj
               endif
               ii=i-1
               jj=j
               if (v(ii,jj).eq.nan) then
                  v(ii,jj)=vel(i,j)
                  stackcount=stackcount+1
                  stacki2(stackcount)=ii
                  stackj2(stackcount)=jj
               endif
               ii=i+1
               jj=j
               if (v(ii,jj).eq.nan) then
                  v(ii,jj)=vel(i,j)
                  stackcount=stackcount+1
                  stacki2(stackcount)=ii
                  stackj2(stackcount)=jj
               endif
               ii=i-1
               jj=j-1
               if (v(ii,jj).eq.nan) then
                  v(ii,jj)=vel(i,j)
                  stackcount=stackcount+1
                  stacki2(stackcount)=ii
                  stackj2(stackcount)=jj
               endif
               ii=i
               jj=j-1
               if (v(ii,jj).eq.nan) then
                  v(ii,jj)=vel(i,j)
                  stackcount=stackcount+1
                  stacki2(stackcount)=ii
                  stackj2(stackcount)=jj
               endif
               ii=i+1
               jj=j-1
               if (v(ii,jj).eq.nan) then
                  v(ii,jj)=vel(i,j)
                  stackcount=stackcount+1
                  stacki2(stackcount)=ii
                  stackj2(stackcount)=jj
               endif
               
            endif
         enddo
         
         count=stackcount
         if (count.gt.0) then
            do k=1,count
               i=stacki2(k)
               j=stackj2(k)
               vel(i,j)=v(i,j)
               stacki(k)=stacki2(k)
               stackj(k)=stackj2(k)
            enddo
         endif
         
      enddo                     ! end of while count...

C Now do the edges
      do i=1,mm
         vel(i,1)=vel(i,2)
         vel(i,nn)=vel(i,nn-1)
      enddo

      do j=1,nn
         vel(1,j)=vel(2,j)
         vel(mm,j)=vel(mm-1,j)
      enddo

      count=0
      do i=1,mm
         do j=1,nn
            count=count+(vel(i,j).eq.nan)
         enddo
      enddo

      if (count.ne.0) then
         print*,'replace has not worked properly...'
         print*, count,'nans found'
         print*,'Calling replace2...'
         CALL replace2(vel,mm,nn,nan)
      endif


      return
      end

C=============================================================
C     fills in regions of NaNs (if the above hasn't worked?!) 
C=============================================================
      subroutine replace2(vel,mm,nn,nan)
      IMPLICIT NONE

      integer*4 mm, nn, M, N
      integer*4 i, j,k, count

      PARAMETER (M=400, N=1600)
      real*4 vel(mm,nn), v(M, N)

      real*4 nan

      do i=1,mm
         do j=1,nn
            v(i,j)=vel(i,j)
         enddo
      enddo

      k=0
      count=1
      do while (count.gt.0)
C      do k=1,2

         do i=2,mm-1
            do j=2,nn-1
               if (vel(i,j).ne.nan) then  
                  
                  if (vel(i-1,j+1).eq.nan) then
                     v(i-1,j+1)=vel(i,j)
                  endif
                  if (vel(i,j+1).eq.nan) then
                     v(i,j+1)=vel(i,j)
                  endif
                  if (vel(i+1,j+1).eq.1) then
                     v(i+1,j+1)=vel(i,j)
                  endif
                  if (vel(i-1,j).eq.nan) then
                     v(i-1,j)=vel(i,j)
                  endif
                  if (vel(i+1,j).eq.nan) then
                     v(i+1,j)=vel(i,j)
                  endif
                  if (vel(i-1,j-1).eq.nan) then
                     v(i-1,j-1)=vel(i,j)
                  endif
                  if (vel(i,j-1).eq.nan) then
                     v(i,j-1)=vel(i,j)
                  endif
                  if (vel(i+1,j-1).eq.nan) then
                     v(i+1,j-1)=vel(i,j)
                  endif
                  
               endif
            enddo
         enddo
         
         count=0
         do i=2,mm-1
            do j=2,nn-1
               vel(i,j)=v(i,j)
               count=count+(vel(i,j).eq.nan)
            enddo
         enddo
         print*, count, ((mm-2)*(nn-2))
         if (count.eq.((mm-2)*(nn-2))) then
            count=-1
         endif
         k=k+1
      enddo                     ! end of while count...

      if (count.eq.-1) then
         print*, 'Input field is all NaNs'
      endif

      return
      end


C=========================================================
C     Gets rid of small regions of data...                           
C=========================================================
      subroutine reduce(vel,size,mm,nn,nan)
      IMPLICIT NONE

      integer*4 mm, nn, M, N, size, total
      integer*4 i, j,ii,jj

      PARAMETER (M=400, N=1600)
      real*4 vel(mm,nn)
      real*4 nan
      
      do i=1,mm-size,2
         do j=1,nn-size,2
            
            total=0
            do ii=0,size
               jj=0
               total=total+(vel(i+ii,j+jj).eq.nan)
               jj=size
               total=total+(vel(i+ii,j+jj).eq.nan)
            enddo
            do jj=1,size-1
               ii=0
               total=total+(vel(i+ii,j+jj).eq.nan)
               ii=size
               total=total+(vel(i+ii,j+jj).eq.nan)
            enddo

            if (total.eq.(size*4)) then ! ie if NaNs all around
               do ii=i,i+size
                  do jj=j,j+size
                     vel(ii,jj)=nan
                  enddo
               enddo
            endif

         enddo
      enddo

      return
      end




